@@ -31,7 +31,6 @@ extern "C" {
 #endif
 
 #include <groonga.h>
-#include "mrn_sys.hpp"
 #include "mrn_mysql_compat.h"
 
 #if (MYSQL_VERSION_ID >= 50603) || \
@@ -264,7 +263,6 @@ private:
   grn_obj **grn_column_ranges;
   grn_obj **grn_index_tables;
   grn_obj **grn_index_columns;
-  bool grn_table_is_referenced;
 
   // buffers
   grn_obj  encoded_key_buffer;
@@ -558,6 +556,13 @@ private:
   int drop_index(MRN_SHARE *target_share, uint key_index);
   grn_obj *find_tokenizer(const char *name, int name_length);
   grn_obj *find_normalizer(KEY *key_info);
+  bool find_token_filters(KEY *key_info, grn_obj *token_filters);
+  bool find_token_filters_put(grn_obj *token_filters,
+                              const char *token_filter_name,
+                              int token_filter_name_length);
+  bool find_token_filters_fill(grn_obj *token_filters,
+                               const char *token_filter_names,
+                               int token_filter_name_length);
   int wrapper_get_record(uchar *buf, const uchar *key);
   int wrapper_get_next_geo_record(uchar *buf);
   int storage_get_next_record(uchar *buf);
@@ -635,6 +640,8 @@ private:
   void storage_store_field_geometry(Field *field,
                                     const char *value, uint value_length);
   void storage_store_field(Field *field, const char *value, uint value_length);
+  void storage_store_field_column(Field *field, bool is_primary_key,
+                                  int nth_column, grn_id record_id);
   void storage_store_fields(uchar *buf, grn_id record_id);
   void storage_store_fields_for_prep_update(const uchar *old_data,
                                             uchar *new_data,
@@ -723,9 +730,6 @@ private:
   int storage_create_indexes(TABLE *table, const char *grn_table_name,
                              grn_obj *grn_table, MRN_SHARE *tmp_share);
   int close_databases();
-  void ensure_database_directory();
-  int ensure_normalizers_register();
-  int ensure_database_create(const char *name);
   int ensure_database_open(const char *name);
   int ensure_database_remove(const char *name);
   int wrapper_delete_table(const char *name, MRN_SHARE *tmp_share,
@@ -737,7 +741,6 @@ private:
   int wrapper_open(const char *name, int mode, uint test_if_locked);
   int wrapper_open_indexes(const char *name);
   int storage_open(const char *name, int mode, uint test_if_locked);
-  void update_grn_table_is_referenced();
   int open_table(const char *name);
   int storage_open_columns(void);
   int storage_open_indexes(const char *name);
@@ -780,7 +783,9 @@ private:
                                               grn_obj *index_column);
   int storage_write_row_multiple_column_indexes(uchar *buf, grn_id record_id);
   int storage_write_row_unique_index(uchar *buf,
-                                     KEY *key_info, grn_obj *index_table,
+                                     KEY *key_info,
+                                     grn_obj *index_table,
+                                     grn_obj *index_column,
                                      grn_id *key_id);
   int storage_write_row_unique_indexes(uchar *buf);
   int wrapper_get_record_id(uchar *data, grn_id *record_id, const char *context);
@@ -899,6 +904,7 @@ private:
                                           grn_obj *match_columns,
                                           uint *consumed_keyword_length,
                                           grn_obj *tmp_objects);
+  grn_expr_flags expr_flags_in_boolean_mode();
   grn_rc generic_ft_init_ext_prepare_expression_in_boolean_mode(
     struct st_mrn_ft_info *info,
     String *key,
@@ -1024,6 +1030,7 @@ private:
   bool storage_is_crashed() const;
   bool wrapper_auto_repair(int error) const;
   bool storage_auto_repair(int error) const;
+  int generic_disable_index(int i, KEY *key_info);
   int wrapper_disable_indexes(uint mode);
   int storage_disable_indexes(uint mode);
   int wrapper_enable_indexes(uint mode);
